@@ -172,6 +172,13 @@ class Protocol(object):
             ' Please use `log_for_times` instead.')
         return self.log_for_times(times)
 
+    def __eq__(self, other):
+        if self is other:
+            return True
+        if not isinstance(other, Protocol):
+            return False
+        return self.code() == other.code()
+
     def events(self):
         """
         Returns a list of all events in this protocol.
@@ -376,9 +383,15 @@ class Protocol(object):
         The time points in the log will be ``a`` and ``b``, and any time in
         between at which the pacing value changes.
 
-        If ``for_drawing`` is set to ``True`` each time value between ``a`` and
-        ``b`` will be listed twice, so that a vertical line can be drawn from
-        the old to the new pacing value.
+        If ``for_drawing`` is set to ``True`` each time value where the
+        protocol changes will be listed twice, so that a vertical line can be
+        drawn from the old to the new pacing value.
+
+        Note that the points returned are from ``a`` to ``b`` inclusive (the
+        interval ``[a, b]``), and so if ``b`` coincides with the end of the
+        protocol a point ``(b, 0)`` will be included in the output (protocol
+        steps are defined as half-open, so include their starting point but not
+        their end point).
         """
         # Test the input
         a, b = float(a), float(b)
@@ -454,6 +467,14 @@ class Protocol(object):
 
         return lo, hi
 
+    def __reduce__(self):
+        """
+        Pickles the Protocol.
+
+        See: https://docs.python.org/3/library/pickle.html#object.__reduce__
+        """
+        return (myokit.parse_protocol, (self.code(), ))
+
     def schedule(self, level, start, duration, period=0, multiplier=0):
         """
         Schedules a new event.
