@@ -301,6 +301,16 @@ class Simulation(myokit.CModule):
         the :class:`myokit.ProgressReporter` interface can be passed in.
         passed in as ``progress``. An optional description of the current
         simulation to use in the ProgressReporter can be passed in as ``msg``.
+
+        The ``duration`` argument cannot be negative, and special care needs to
+        be taken when very small (positive) values are used. If ``duration`` is
+        zero or so small that
+        ``simulation.time() + duration == simulation.time()``, then the method
+        returns without updating the internal states or time. However, for
+        some extremely short durations (approx ``2 epsilon * time``), the
+        simulation will try to run but the underlying CVODE engine may return a
+        ``CV_TOO_CLOSE`` error, causing a :class:`myokit.SimulationError` to be
+        raised.
         """
         duration = float(duration)
         output = self._run(
@@ -377,17 +387,26 @@ class Simulation(myokit.CModule):
             bench = None
 
         # Run simulation
+        # The simulation is run only if (tmin + duration > tmin). This is a
+        # stronger check than (duration == 0), which will return true even for
+        # very short durations (and will cause zero iterations of the
+        # "while (t < tmax)" loop below).
+
         with myokit.SubCapture() as capture:
-            if duration > 0:
+            istate = list(self._state)
+            if tmin + duration > tmin:
+
+                # Lists to return state in
+                rstate = list(istate)
+                rbound = [0, 0, 0, 0]  # time, pace, realtime, evaluations
+
                 # Initialize
-                state = [0] * len(self._state)
-                bound = [0, 0, 0, 0]    # time, pace, realtime, evaluations
                 self._sim.sim_init(
                     tmin,
                     tmax,
-                    list(self._state),
-                    state,
-                    bound,
+                    istate,
+                    rstate,
+                    rbound,
                     self._protocol,
                     self._fixed_form_protocol,
                     log,
@@ -398,6 +417,7 @@ class Simulation(myokit.CModule):
                     bench,
                 )
                 t = tmin
+
                 try:
                     if progress:
                         # Allow progress reporters to bypass the subcapture
@@ -413,27 +433,29 @@ class Simulation(myokit.CModule):
                         # Loop without feedback
                         while t < tmax:
                             t = self._sim.sim_step()
-                except ArithmeticError:
-                    self._error_state = list(state)
+                except ArithmeticError as e:
+                    # Some CVODE errors are set to raise an ArithmeticError,
+                    # which users may be able to debug.
+                    self._error_state = list(rstate)
                     txt = ['A numerical error occurred during simulation at'
                            ' t = ' + str(t) + '.', 'Last reached state: ']
-                    txt.extend([
-                        '  ' + x for x
-                        in self._model.format_state(state).splitlines()])
+                    txt.extend(['  ' + x for x in
+                                self._model.format_state(rstate).splitlines()])
                     txt.append('Inputs for binding: ')
-                    txt.append('  time        = ' + myokit.strfloat(bound[0]))
-                    txt.append('  pace        = ' + myokit.strfloat(bound[1]))
-                    txt.append('  realtime    = ' + myokit.strfloat(bound[2]))
-                    txt.append('  evaluations = ' + myokit.strfloat(bound[3]))
+                    txt.append('  time        = ' + myokit.strfloat(rbound[0]))
+                    txt.append('  pace        = ' + myokit.strfloat(rbound[1]))
+                    txt.append('  realtime    = ' + myokit.strfloat(rbound[2]))
+                    txt.append('  evaluations = ' + myokit.strfloat(rbound[3]))
+                    txt.append(str(e))
                     try:
-                        self._model.eval_state_derivatives(state)
+                        self._model.eval_state_derivatives(rstate)
                     except myokit.NumericalError as en:
                         txt.append(str(en))
                     raise myokit.SimulationError('\n'.join(txt))
                 except Exception as e:
 
                     # Store error state
-                    self._error_state = list(state)
+                    self._error_state = list(rstate)
 
                     # Check for known CVODE errors
                     if 'Function CVode()' in str(e):
@@ -451,8 +473,9 @@ class Simulation(myokit.CModule):
                 finally:
                     # Clean even after KeyboardInterrupt or other Exception
                     self._sim.sim_clean()
+
                 # Update internal state
-                self._state = state
+                self._state = rstate
 
         # Return
         if root_list is not None:
