@@ -59,6 +59,7 @@ class JacobianTracer(myokit.CppModule):
         # Extension module id
         JacobianTracer._index += 1
         module_name = 'myokit_JacobianTracer_' + str(JacobianTracer._index)
+        module_name += '_' + str(myokit._pid_hash())
 
         # Template arguments
         args = {
@@ -235,8 +236,9 @@ class JacobianCalculator(myokit.CppModule):
 
         # Extension module id
         JacobianCalculator._index += 1
-        module_name = 'myokit_JacobianCalculator_' \
-            + str(JacobianCalculator._index)
+        module_name = 'myokit_JacobianCalculator_'
+        module_name += str(JacobianCalculator._index)
+        module_name += '_' + str(myokit._pid_hash())
 
         # Template arguments
         args = {
