@@ -21,7 +21,7 @@ KERNEL_FILE = 'openclsim.cl'
 
 class SimulationOpenCL(myokit.CModule):
     """
-    Can run 1d or 2d simulations based on a :class:`model <Model>` using
+    Can run multi-cellular simulations based on a :class:`model <Model>` using
     OpenCL for parallelization.
 
     Takes the following input arguments:
@@ -97,7 +97,16 @@ class SimulationOpenCL(myokit.CModule):
     allows you to pre-pace, run a simulation, reset to the pre-paced state, run
     another simulation etc.
 
-    The ``diffusion_current`` is calculated as::
+    To set up a 1d simulation, the argument ``ncells`` should be given as a
+    tuple. In this case, any cell ``i`` will be assumed to be connected to
+    cells ``i - 1`` and ``i + 1`` (except at the boundaries).
+    Similarly, if ``ncells`` is a 2-dimensional tuple ``(nx, ny)`` a grid will
+    be assumed so that each (non-boundary) cell is connected to four
+    neighbours.
+    Finally, arbitrary geometries can be used by passing a scalar to ``ncells``
+    and specifying the connections with :meth:`set_connections`.
+
+    The ``diffusion_current`` in any cell is calculated as::
 
         i = sum[g * (V - V_j)]
 
@@ -376,29 +385,62 @@ class SimulationOpenCL(myokit.CModule):
             return self._gx
         return (self._gx, self._gy)
 
-    def find_nan(self, log, watch_var=None, safe_range=None):
+    def find_nan(self, log, watch_var=None, safe_range=None, return_log=False):
         """
-        Searches for the origin of a ``NaN`` (or ``inf``) in a simulation log
-        generated by this Simulation.
+        Searches for the origin of a bad value (``NaN`` or ``inf``) in a data
+        log generated by this simulation.
 
-        The log must contain the state of each cell and all bound variables.
-        The NaN can occur at any point in time except the first.
+        Arguments:
 
-        Returns a tuple ``(time, icell, variable, value, states, bound)`` where
-        ``time`` is the time the first ``NaN`` was found and ``icell`` is the
-        index of the cell in which it happened. The variable's name is given as
-        ``variable`` and its (illegal) value as ``value``. The current state
-        and, if available, any previous states are given in the list
-        ``states``. Here, ``states[0]`` points to the current state,
-        ``state[1]`` is the previous state and so on. Similarly the values of
-        the model's bound variables is given in ``bound``.
+        ``log``
+            A :class:`myokit.DataLog` from this simulation. The log must
+            contain the state of each cell and all bound variables. The bad
+            value can occur at any point in time except the first.
+        ``watch_var``
+            To aid in diagnosis, a variable can be selected as ``watch_var``
+            and a ``safe_range`` can be specified. With this option, the
+            function will find and report either the first bad value or the
+            first time the watched variable left the safe range, whatever came
+            first. The watched variable must be a state variable.
+        ``safe_range``
+            The safe range to check the ``watch_var`` against. The safe range
+            should be specified as ``(lower, upper)`` where both bounds are
+            assumed to be in the safe range.
+        ``return_log``
+            If set to ``True``, a log containing the final points before the
+            error occurred will be returned, containing all variables for all
+            cells.
+
+        Returns a tuple containing the following six values (or seven values if
+        ``return_log=True``):
+
+        ``time``
+            The time the first bad value was found.
+        ``icell``
+            The index of the cell in which the bad value was found, as a tuple
+            e.g. ``(3, )`` in a 1d simulation or ``(15, 12)`` in a 2d
+            simulation.
+        ``variable``
+            The name of the variable that was detected to be ``NaN`` or
+            ``inf``.
+        ``value``
+            The bad value detected)
+        ``states``
+            The state at time ``time`` and, if possible, up to 3 earlier
+            states. Here, ``states[0]`` points to the current state,
+            ``state[1]`` is the previous state and so on. Each state is
+            represented as a list of values.
+        ``bounds``
+            The bound variables corresponding to the returned ``states``. For
+            ``diffusion_current``, this will contain the specific cell's
+            current, for ``time`` and ``pace`` the global values are shown.
+            Each entry in ``bounds`` is a dictionary from variable names to
+            values.
+        ``log``
+            A :class:`myokit.DataLog` with all variables for the same points as
+            ``states`` and ``bounds``. This will only be included if
+            ``return_log=True``.
 
-        To aid in diagnosis, a variable can be selected as ``watch_var`` and a
-        ``safe_range`` can be specified. With this option, the function will
-        find and report either the first ``NaN`` or the first time the watched
-        variable left the safe range, whatever came first. The safe range
-        should be specified as ``(lower, upper)`` where both bounds are assumed
-        to be in the safe range. The watched variable must be a state variable.
         """
         import numpy as np
 
@@ -437,18 +479,18 @@ class SimulationOpenCL(myokit.CModule):
 
             def find_error_position(log):
                 # Search for first occurrence of propagating NaN in the log
-                ifirst = None
-                kfirst = None
+                ifirst = None   # Index in time
+                kfirst = None   # Variable + cell index
                 for key, ar in log.items():
                     if ifirst is None:
                         if not np.isfinite(ar[-1]):
-                            # First NaN found
+                            # NaN found in the log
                             kfirst = key
                             ifirst = bisect(ar, 0, len(ar) - 1)
                             if ifirst == 0:
                                 break
                     elif not np.isfinite(ar[ifirst - 1]):
-                        # Earlier NaN found
+                        # Earlier NaN found than before
                         kfirst = key
                         ifirst = bisect(ar, 0, ifirst)
                         if ifirst == 0:
@@ -587,15 +629,17 @@ class SimulationOpenCL(myokit.CModule):
             return s, b
 
         # Get error cell's states before, during and after
+        times = []
         states = []
-        bound = []
+        bounds = []
         max_states = 3
         for k in range(ifirst, ifirst - max_states - 1, -1):
             if k < 0:
                 break
+            times.append(log[time_var][k])
             s, b = state(k, icell)
             states.append(s)
-            bound.append(b)
+            bounds.append(b)
 
         # Get variable causing error
         var = self._model.get('.'.join(kfirst.split('.')[ndims:]))
@@ -607,17 +651,134 @@ class SimulationOpenCL(myokit.CModule):
         # Get time error occurred
         time = log[time_var][ifirst]
 
-        # Return time, icell, variable, value, states, bound
-        return time, icell, var, value, states, bound
+        # Get all variables at shown states
+        if return_log:
+            # Get earliest state in states/bounds
+            state = []
+            istart = max(0, ifirst - max_states)
+            for dims in myokit._dimco(*self._dims):
+                pre = '.'.join([str(x) for x in dims]) + '.'
+                for s in self._model.states():
+                    state.append(log[pre + s.qname()][istart])
+
+            # Save current state & time, and rewind
+            old_state = self._state
+            old_time = self._time
+            self._state = state
+            self._time = times[-1]
+
+            # Run for all states
+            duration = len(times) * self._step_size
+            log = self.run(
+                duration,
+                log=myokit.LOG_BOUND + myokit.LOG_STATE + myokit.LOG_INTER,
+                log_interval=0,
+                report_nan=False)
+
+            # Reset simulation to original state
+            self._state = old_state
+            self._time = old_time
+
+            # Return
+            return time, icell, var, value, states, bounds, log
+
+        return time, icell, var, value, states, bounds
 
     def is2d(self):
-        """
-        Returns True if and only if this is a 2d simulation.
-        """
+        """Deprecated alias of :meth:`is_2d()`."""
+        # Deprecated since 2020-09-10
+        import warnings
+        warnings.warn('The method SimulationOpenCL.is2d() is deprecated.'
+                      ' Please use is_2d() instead.')
+        return self.is_2d()
+
+    def is_2d(self):
+        """Returns ``True`` if and only if this is a 2d simulation. """
         return len(self._dims) == 2
 
-    def pre(
-            self, duration, report_nan=True, progress=None,
+    def is_paced(self, x, y=None):
+        """
+        Returns ``True`` if and only if the cell at index ``x`` (or index
+        ``(x, y)`` in 2d simulations) will be paced during simulations.
+        """
+        # Check input
+        x = int(x)
+        if x < 0 or x >= self._dims[0]:
+            raise ValueError('X-coordinate out of range: ' + str(x) + '.')
+        if len(self._dims) == 2:
+            if y is None:
+                raise ValueError(
+                    'No y-coordinate specified in 2-dimensional simulation.')
+            y = int(y)
+            if y < 0 or y >= self._dims[1]:
+                raise ValueError('Y-coordinate out of range: ' + str(y) + '.')
+        else:
+            if not (y is None or y == 0):
+                raise ValueError(
+                    'Y-coordinate specified in 1-dimensional simulation.')
+            y = 0
+
+        # Pacing rectangle
+        if type(self._paced_cells) == tuple:
+            nx, ny, xmin, ymin = self._paced_cells
+            return (x >= xmin and x < xmin + nx and
+                    y >= ymin and y < ymin + ny)
+
+        # Explicit cell selection
+        cid = x + y * self._dims[0]
+        return cid in self._paced_cells
+
+    def neighbours(self, x, y=None):
+        """
+        Returns a list of indices specifying the neighbours of the cell at
+        index ``x`` (or index ``(x, y)`` for 2d simulations).
+
+        Indices are given either as integers (1d or arbitrary geometry) or as
+        tuples (2d).
+        """
+        # Check input
+        x = int(x)
+        if x < 0 or x >= self._dims[0]:
+            raise ValueError('X-coordinate out of range: ' + str(x) + '.')
+        if len(self._dims) == 2:
+            if y is None:
+                raise ValueError(
+                    'No y-coordinate specified in 2-dimensional simulation.')
+            y = int(y)
+            if y < 0 or y >= self._dims[1]:
+                raise ValueError('Y-coordinate out of range: ' + str(y) + '.')
+        else:
+            if not (y is None or y == 0):
+                raise ValueError(
+                    'Y-coordinate specified in 1-dimensional simulation.')
+
+        # User-specified connections (always 1d)
+        if self._connections is not None:
+            neighbours = []
+            for i, j, c in self._connections:
+                if i == x:
+                    neighbours.append(j)
+                elif j == x:
+                    neighbours.append(i)
+            return neighbours
+
+        # Left and right neighbours
+        neighbours = []
+        if x > 0:
+            neighbours.append(x - 1)
+        if x + 1 < self._dims[0]:
+            neighbours.append(x + 1)
+
+        # Top and bottom neighbours
+        if len(self._dims) == 2:
+            neighbours = [(i, y) for i in neighbours]
+            if y > 0:
+                neighbours.append((x, y - 1))
+            if y + 1 < self._dims[1]:
+                neighbours.append((x, y + 1))
+        return neighbours
+
+    def pre(self, duration, report_nan=True, progress=None,
             msg='Pre-pacing SimulationOpenCL'):
         """
         This method can be used to perform an unlogged simulation, typically to
@@ -671,8 +832,7 @@ class SimulationOpenCL(myokit.CModule):
         self._time = 0
         self._state = list(self._default_state)
 
-    def run(
-            self, duration, log=None, log_interval=1.0, report_nan=True,
+    def run(self, duration, log=None, log_interval=1.0, report_nan=True,
             progress=None, msg='Running SimulationOpenCL'):
         """
         Runs a simulation and returns the logged results. Running a simulation
@@ -883,28 +1043,127 @@ class SimulationOpenCL(myokit.CModule):
             txt = ['Numerical error found in simulation logs.']
             try:
                 # NaN encountered, show how it happened
-                time, icell, var, value, states, bound = self.find_nan(log)
+                time, icell, var, value, states, bounds, d = self.find_nan(
+                    log, return_log=True)
+                icell_str = '(' + ','.join([str(x) for x in icell]) + ')'
                 txt.append(
-                    'Encountered numerical error at t=' + str(time)
-                    + ' in cell (' + ','.join([str(x) for x in icell])
-                    + ') when ' + var + '=' + str(value) + '.')
+                    'Encountered numerical error at t='
+                    + myokit.strfloat(time, precision=self._precision)
+                    + ' in cell ' + icell_str + ' when ' + var + '='
+                    + myokit.strfloat(value, precision=self._precision) + '.')
+
+                # Check if this cell was paced.
+                is_paced = self.is_paced(*icell)
+                is_paced_str = (', cell ' + icell_str + ' '
+                                + ('IS' if is_paced else 'is NOT') + ' paced')
+
+                # Get the names of the variables used in bindings, to use when
+                # calculating the derivatives
+                vtime = self._model.time().qname()
+                vpace = self._model.binding('pace')
+                vpace = None if vpace is None else vpace.qname()
+                vdiff = None
+                if self._diffusion_enabled:
+                    vdiff = self._model.binding('diffusion_current')
+                    vdiff = None if vdiff is None else vdiff.qname()
+
+                # List all neighbours for this cell
+                neighbours = self.neighbours(*icell)
+                if len(self._dims) == 2:
+                    neighbours_str = ', '.join(
+                        ('(' + ','.join([str(i) for i in j]) + ')')
+                        for j in neighbours)
+                else:
+                    neighbours_str = ', '.join(str(j) for j in neighbours)
+
+                # Show final state
+                txt.append('State during:')
+                txt.append(self._model.format_state(
+                    states[0], precision=self._precision))
+
+                # Show bound variables
+                txt.append('Simulation variables during:')
+                txt.append('  Time: ' + myokit.strfloat(
+                    bounds[0][vtime], precision=self._precision))
+                if vpace:
+                    txt.append(
+                        '  Pacing variable: ' + myokit.strfloat(
+                            bounds[0][vpace], precision=self._precision)
+                        + is_paced_str)
+                if vdiff:
+                    txt.append(
+                        '  Diffusion current: ' + myokit.strfloat(
+                            bounds[0][vdiff], precision=self._precision))
+                if neighbours:
+                    txt.append('  Connected cells: ' + neighbours_str)
+
+                # Show previous state (and derivatives)
                 n_states = len(states)
                 txt.append('Obtained ' + str(n_states) + ' previous state(s).')
                 if n_states > 1:
-                    txt.append('State before:')
-                    txt.append(self._model.format_state(states[1]))
-                txt.append('State during:')
-                txt.append(self._model.format_state(states[0]))
-                if n_states > 1:
-                    txt.append('Evaluating derivatives at state before...')
+                    # Get state and input at previous state
+                    state = states[1]
+                    bound = {
+                        'time': bounds[1][vtime],
+                        'pace': 0,
+                        'diffusion_current': 0,
+                    }
+                    if is_paced and vpace is not None:
+                        bound['pace'] = bounds[1][vpace]
+                    if vdiff is not None:
+                        bound['diffusion_current'] = bounds[1][vdiff]
+
+                    # Evaluate state derivatives
+                    eval_error = None
                     try:
                         derivs = self._model.eval_state_derivatives(
-                            states[1], precision=self._precision)
-                        txt.append(
-                            self._model.format_state_derivatives(
-                                states[1], derivs))
+                            state, bound, self._precision, ignore_errors=False)
                     except myokit.NumericalError as ee:
-                        txt.append(str(ee))
+                        derivs = self._model.eval_state_derivatives(
+                            state, bound, self._precision, ignore_errors=True)
+                        eval_error = str(ee)
+
+                    # Show state and derivatives
+                    txt.append('State before:')
+                    txt.append(self._model.format_state_derivatives(
+                        state, derivs, self._precision))
+
+                    # Show bound variables
+                    txt.append('Simulation variables before:')
+                    txt.append('  Time: ' + myokit.strfloat(
+                        bounds[1][vtime], precision=self._precision))
+                    if vpace:
+                        txt.append(
+                            '  Pacing variable: ' + myokit.strfloat(
+                                bounds[1][vpace], precision=self._precision)
+                            + is_paced_str)
+                    if vdiff:
+                        txt.append(
+                            '  Diffusion current: ' + myokit.strfloat(
+                                bounds[1][vdiff], precision=self._precision))
+                    if neighbours:
+                        txt.append('  Connected cells: ' + neighbours_str)
+
+                    # Show all variables with non-finite values
+                    txt.append(
+                        'Logged all variables for points: ' + ', '.join(
+                            myokit.strfloat(t, precision=self._precision)
+                            for t in d.time()))
+                    txt.append(
+                        'Non-finite valued variables at t=' + myokit.strfloat(
+                            d.time()[-2], precision=self._precision))
+                    for key, values in d.items():
+                        value = values[-2]
+                        if not np.isfinite(value):
+                            txt.append(
+                                '  ' + str(key) + ' = ' + myokit.strfloat(
+                                    value, precision=self._precision))
+
+                    # Show any error in evaluating derivatives
+                    if eval_error is not None:
+                        txt.append('Error when evaluating derivatives:')
+                        txt.append(eval_error)
+
             except myokit.FindNanError as e:
                 txt.append(
                     'Unable to pinpoint source of NaN, an error occurred:')
@@ -964,20 +1223,28 @@ class SimulationOpenCL(myokit.CModule):
                 raise ValueError(
                     'Connections must be None or a list of 3-tuples'
                     ' (cell_index_1, cell_index_2, conductance).')
+
+            # Check indices
             i, j = int(i), int(j)
             if i == j or i < 0 or j < 0 or i >= self._nx or j >= self._nx:
                 raise ValueError(
                     'Invalid connection: (' + str(i) + ', ' + str(j) + ', '
                     + str(c) + ')')
+
+            # Order indices, and detect doubles
             i, j = (i, j) if i < j else (j, i)
             if (i, j) in doubles:
                 raise ValueError(
                     'Duplicate connection: (' + str(i) + ', ' + str(j) + ', '
                     + str(c) + ')')
             doubles.add((i, j))
+
+            # Check conductance
             c = float(c)
             if c < 0:
                 raise ValueError('Invalid conductance: ' + str(c))
+
+            # Store connection
             conns.append((i, j, c))
         del(doubles)
         self._connections = conns
@@ -1095,8 +1362,12 @@ class SimulationOpenCL(myokit.CModule):
         # Check nx and x. Allow cell selections outside of the boders!
         nx = int(nx)
         x = int(x)
+        if nx < 0:
+            nx = -nx
+            x -= nx
         if x < 0:
             x += self._nx
+
         # Check dimensions
         if len(self._dims) == 1:
             # Use default for y
@@ -1105,8 +1376,12 @@ class SimulationOpenCL(myokit.CModule):
         else:
             # Check ny and y
             ny = int(ny)
+            if ny < 0:
+                ny = -ny
+                y -= ny
             if y < 0:
                 y += self._ny
+
         # Set tuple of paced cells
         self._paced_cells = (nx, ny, x, y)
 
@@ -1228,7 +1503,7 @@ class SimulationOpenCL(myokit.CModule):
 
         If the optional arguments ``x`` and ``y`` specify a valid cell index a
         single cell's state is returned. For example ``state(4)`` can be
-        used with a 1d simulation, while ``state(4,2)`` is a valid index in
+        used with a 1d simulation, while ``state(4, 2)`` is a valid index in
         the 2d case.
         """
         if x is None:
